// CBezier.cpp


#include "StdAfx.h"
#include <math.h>
#include "Bezier.h"
#include <memory.h>

#define HSZ	3	//half rect point size

IMPLEMENT_DYNAMIC(CBezier, CObject)

CBezier::CBezier()
{
		m_nWidth = 1;
		m_bClosed = false;
		m_lpPoints = NULL; //new CPoint[7];
		m_curvePen.CreatePen(PS_SOLID, m_nWidth, RGB(0,0,0));
		m_handlePen.CreatePen(PS_DOT,1,RGB(0,0,255));
		m_ForeColor=RGB(0,0,0);
		m_BackColor=RGB(255,255,255);
		m_bCaptured = false;
		m_nAmount = 0;
		m_bShowHandles = false;
		m_rgn.CreateRectRgn(0, 0, 300, 300);
		m_mode = VIEW;
}

CBezier::~CBezier()
{
	if (m_lpPoints) delete[] m_lpPoints;
}

double CBezier::CalcSlope(long p0, long p1, long p2, long p3, double u)
{
	double u2 = u*u;

	return (3 * (-p0 + p1 + 2 * p0 * u - 4 * p1 * u + 2 * p2 * u - 
		p0 * u2 + 3 * p1 * u2 - 3 * p2 * u2 + p3 * u2));
}

void CBezier::CalcHandle(int nIndex, int dc)
{
	long x0, y0, x1, y1, dx, dy;
	double dxc, dyc;

	x0 = m_lpPoints[nIndex].x;
	y0 = m_lpPoints[nIndex].y;

	x1 = m_lpPoints[nIndex + 1].x;
	y1 = m_lpPoints[nIndex + 1].y;

	dx = x1 - x0;
	dy = y1 - y0;

	dxc = (sqrt (dx*dx + dy*dy) - dc) / sqrt((dy / dx)*(dy / dx) + 1);
	dyc = dxc * dy / dx;

	m_lpPoints[nIndex + 1].x = (long) dxc + x0;
	m_lpPoints[nIndex + 1].y = (long) dyc + y0;
}

double CBezier::CalcPto(long p0, long p1, long p2, long p3, double u)
{
	double t = 1 - u;
	//return (pow(1-u,3)*p0 + 3*pow(1-u,2)*u*p1 + 3 * (1-u)*pow(u,2)*p2 + pow(u,3)*p3);
	return ((t*t*t)*p0 + 3*(t*t)*u*p1 + 3 * t*(u*u)*p2 + (u*u*u)*p3);
}

void CBezier::CalcMinMax (long p0, long p1, long p2, long p3, long& lMin, long& lMax)
{
	double b, aux;	//aux es por el problema de pow en modo release

	if ( p0 < p3) {
		lMin = p0;
		lMax = p3;
	}
	else {
		lMin = p3;
		lMax = p0;
	}

	aux = (2 * p0 - 4 * p1 + 2 * p2);
	b = aux * aux - 4 * (-p0 + p1) * (-p0 + 3 * p1 - 3 * p2 + p3);

	if(b >= 0) {
		double a, c, u;
		long lPc1, lPc2;

		a = -2 * p0 + 4 * p1 - 2 * p2;
		b = sqrt(b);
		c = 2 * (-p0 + 3 * p1 - 3 * p2 + p3);

		if ( (u = (a + b) / c) >= 0 && u <= 1) {
			lPc1 = (long) CalcPto(p0, p1, p2, p3, u);
			if (lPc1 < lMin) lMin = lPc1;		
			else if (lPc1 > lMax) lMax = lPc1;
		}
		if ( (u = (a - b) / c) >= 0 && u <= 1) {
			lPc2 = (long) CalcPto(p0, p1, p2, p3, u);
			if (lPc2 < lMin) lMin = lPc2;
			else if (lPc2 > lMax) lMax = lPc2;
		}
	}
	lMin -= 3;	//because frame or round error
	lMax += 3;
}

void CBezier::DoRect(RECT* pRect,long l, long t, long dr, long db)
{
	pRect->left = l;
	pRect->top = t;
	pRect->right = l + dr;
	pRect->bottom = t + db;
}

void CBezier::AddPoint(long x, long y)
{
	CPoint* pNew = new CPoint[m_nAmount + 1];
	if (pNew) {
		if (m_lpPoints) {
			memcpy(pNew, m_lpPoints, m_nAmount * sizeof(CPoint));
			delete[] m_lpPoints;
		}
		m_lpPoints = pNew;
		
		/* copy new point */
		m_lpPoints[m_nAmount].x = x;
		m_lpPoints[m_nAmount].y = y;
		m_nAmount++;
	}
}

bool CBezier::AddControlPoint(const CPoint& pt, int nIndex, double u)
{	
	long dx, dy;
	double m;
	bool flag;
	int num = nIndex + 2;
	
	ASSERT(IsControl(nIndex));

	if (m_nAmount == 0) {
		m_lpPoints = new CPoint[1];
		if(!m_lpPoints) return false;
		m_nAmount = 1;
		*m_lpPoints = pt;
	}
	else if (m_nAmount == 1) {
		CPoint* pNew = new CPoint[4];
		if(!pNew) return false;
		*pNew = *m_lpPoints;
		delete[] m_lpPoints;
		m_lpPoints = pNew;		
		m_nAmount = 4;
		m_lpPoints[1].x = m_lpPoints->x + 30;
		m_lpPoints[1].y = m_lpPoints->y + 30;
		m_lpPoints[2].x = pt.x - 30;
		m_lpPoints[2].y = pt.y - 30;
		m_lpPoints[3] = pt;
	}
	else {
		CPoint* p = m_lpPoints + nIndex;			

		dx = (long) CalcSlope(p->x, p[1].x, p[2].x, p[3].x, u);
		dy = (long) CalcSlope(p->y, p[1].y, p[2].y, p[3].y, u);

		(dx < 0) ? flag = true: flag = false;

		m = dy / dx;		// calc the slope
		dx = (long) (20 / sqrt(m * m + 1));
		dy = (long) (m * dx);
				
		CPoint* pNew = new CPoint[m_nAmount + 3];

		if (!pNew) return false;
		
		if (m_lpPoints) {
			memcpy(pNew, m_lpPoints, num * sizeof(CPoint));
			memcpy(pNew + num + 3, m_lpPoints + num, (m_nAmount - num) * sizeof(CPoint));
			delete[] m_lpPoints;
		}
		m_lpPoints = pNew;

		if (flag) {
			dx *= -1;
			dy *= -1;
		}

		// copy new points
		m_lpPoints[num].x = (long) (pt.x - dx);
		m_lpPoints[num++].y = (long) (pt.y - dy);
		m_nAmount++;
		
		m_lpPoints[num].x = pt.x;
		m_lpPoints[num++].y = pt.y;
		m_nAmount++;
		
		m_lpPoints[num].x = (long) (pt.x + dx);
		m_lpPoints[num++].y = (long) (pt.y + dy);
		m_nAmount++;
	}

	return true;
}

void CBezier::AddFirstPoint(long x, long y)
{
	if (m_lpPoints) delete[] m_lpPoints;	//borra los puntos existentes
	if (m_lpPoints = new CPoint) {
		m_nAmount = 1;
		m_lpPoints->x = x;
		m_lpPoints->y = y;
	}
}

void CBezier::AddCurve(long x1, long y1, long x2, long y2, long x3, long y3)
{
	CPoint* pNew = new CPoint[m_nAmount + 3];

	if (pNew) {
		if (m_lpPoints) {
			memcpy(pNew, m_lpPoints, m_nAmount * sizeof(CPoint));
			delete[] m_lpPoints;
		}
		m_lpPoints = pNew;
		
		/* copy new points */
		m_lpPoints[m_nAmount].x = x1;
		m_lpPoints[m_nAmount].y = y1;
		m_nAmount++;
		m_lpPoints[m_nAmount].x = x2;
		m_lpPoints[m_nAmount].y = y2;
		m_nAmount++;
		m_lpPoints[m_nAmount].x = x3;
		m_lpPoints[m_nAmount].y = y3;
		m_nAmount++;
	}
}

void CBezier::Close()
{
	if (!m_bClosed && m_nAmount > 1) {
		if ((m_nAmount % 3) == 0) {
			AddPoint(0, 0); // no importan los valores
		}
		ASSERT((m_nAmount % 3) != 2);

		m_lpPoints[m_nAmount - 1].x = m_lpPoints->x;
		m_lpPoints[m_nAmount - 1].y = m_lpPoints->y;
		m_bClosed = true;
	}
}

void CBezier::AddCurveRgn(int i)
{
	if (m_bClosed) {
		if (i < 0) i = m_nAmount + i;
		else if (i >= m_nAmount) i = m_nAmount - i;
	}
	if ( i >= 0 && i < m_nAmount - 1) {
		CRgn rgn;
		long left, top, right, bottom;
		CPoint* p = m_lpPoints + i - (i % 3);	//could be a handle
		
		CalcMinMax(p[0].x, p[1].x, p[2].x, p[3].x, left, right);
		CalcMinMax(p[0].y, p[1].y, p[2].y, p[3].y, top, bottom);

		rgn.CreateRectRgn(left, top, right, bottom);
		m_rgn.CombineRgn(&rgn, &m_rgn, RGN_OR);
	}
}

void CBezier::AddHandleRgn(int i)
{
	if (m_bClosed) {
		if (i < 0) i = m_nAmount + i;
		else if (i >= m_nAmount) i = m_nAmount - i;
	}

	int pos = i % 3;

	if ( pos > 0 && i < m_nAmount) {
		CRgn rgn;
		long left, top, right, bottom;
		CPoint *p1, *p2;
		
		p1 = m_lpPoints + i;
		if ( pos == 1) p2 = p1 - 1;
		else p2 = p1 + 1;

		if (p1->x < p2->x) { left = p1->x - HSZ; right = p2->x + HSZ; }
		else { left = p2->x - HSZ; right = p1->x + HSZ; }
		
		if (p1->y < p2->y) { top = p1->y - HSZ; bottom = p2->y + HSZ; }
		else { top = p2->y - HSZ; bottom = p1->y + HSZ; }

		rgn.CreateRectRgn(left, top, right, bottom);
		m_rgn.CombineRgn(&rgn, &m_rgn, RGN_OR);
	}
}

bool CBezier::DelControlPoint(const CPoint& pt, int nIndex)
{	
	ASSERT(m_lpPoints);
	ASSERT(IsControl(nIndex));

	if (m_nAmount == 3) {
		delete[] m_lpPoints;
		m_lpPoints = NULL;
	}
	else {
		CPoint* pNew = new CPoint[m_nAmount - 3];
				
		if (!pNew) return false;

		if (nIndex == 0) {
			memcpy(pNew, m_lpPoints + 3, (m_nAmount - 3) * sizeof(CPoint));
			if (m_bClosed) {
				pNew[m_nAmount - 4] = *pNew;
			}
		}
		else if (nIndex == m_nAmount - 1) {
			memcpy(pNew, m_lpPoints, (m_nAmount - 3) * sizeof(CPoint));
			if (m_bClosed) {
				*pNew = pNew[m_nAmount - 4];
			}
		}
		else {
			memcpy(pNew, m_lpPoints, nIndex * sizeof(CPoint));
			memcpy(pNew + nIndex, m_lpPoints + nIndex + 3, 
				(m_nAmount - nIndex - 3) * sizeof(CPoint));
		}

		delete[] m_lpPoints;		
		m_lpPoints = pNew;
	}
	m_nAmount -= 3;
	return true;
}

void CBezier::GetCurve(int nCurve, CPoint* p1, CPoint* p2, CPoint* p3)
{
	int i = nCurve * 3;

	p1->x = m_lpPoints[i].x;
	p1->y = m_lpPoints[i++].y;
	p2->x = m_lpPoints[i].x;
	p2->y = m_lpPoints[i++].y;
	p3->x = m_lpPoints[i].x;
	p3->y = m_lpPoints[i++].y;
}

bool CBezier::IsControlPoint(const CPoint& pt, int * nIndex)
{	
	for (int i = 0; i < m_nAmount; i += 3) {
		if (pt.x >= m_lpPoints[i].x - HSZ && pt.x <= m_lpPoints[i].x + HSZ &&
			pt.y >= m_lpPoints[i].y - HSZ && pt.y <= m_lpPoints[i].y + HSZ ) {
			*nIndex = i;
			return true;
		}
	}
	return false;
}

bool CBezier::IsHandlePoint(const CPoint& pt, int * nIndex)
{	
	for (int i = 1; i < m_nAmount; i += 2) {
		if (pt.x >= m_lpPoints[i].x - HSZ && pt.x <= m_lpPoints[i].x + HSZ &&
			pt.y >= m_lpPoints[i].y - HSZ && pt.y <= m_lpPoints[i].y + HSZ ) {
			*nIndex = i;
			return true;
		}
		i++;
		if (pt.x >= m_lpPoints[i].x - HSZ && pt.x <= m_lpPoints[i].x + HSZ &&
			pt.y >= m_lpPoints[i].y - HSZ && pt.y <= m_lpPoints[i].y + HSZ ) {
			*nIndex = i;
			return true;
		}
	}
	return false;
}

bool CBezier::IsCurvePoint(const CPoint& pt, int* nIndex, double* dU, CPoint* onCurve)
{
	int i;
	long x, y;
	double u;
	CPoint* p;

	/////////////////////////
	//CString str;
	//str.Format("(%d,%d)", pt.x, pt.y);
	//AfxMessageBox(str);
	//CPoint pt;
	//pt.x = 189;
	//pt.y = 231;
	////////////////////////

	for (i = 0, p = m_lpPoints; i < m_nAmount; i += 3, p += 3) {
		for (u = 0; u < 1; u += 0.001) {
			x = (long) CalcPto(p->x, p[1].x, p[2].x, p[3].x, u);
			y = (long) CalcPto(p->y, p[1].y, p[2].y, p[3].y, u);

			//TRACE("\n(%d,%d)(%d,%d)(%d,%d)(%d,%d) = (%d,%d)",p->x, p->y, p[1].x, p[1].y, p[2].x, p[2].y, p[3].x, p[3].y, x, y);
			//str.Format("(%d,%d)(%d,%d)(%d,%d)(%d,%d) = (%d,%d)",p->x, p->y, p[1].x, p[1].y, p[2].x, p[2].y, p[3].x, p[3].y, x, y);
			//AfxMessageBox(str);
			
			if (pt.x >= x-2 && pt.x <= x + 2 
				&& pt.y >= y-2 && pt.y <= y+2) 
			{		
				*nIndex = i;
				*dU = u;
				onCurve->x = x;
				onCurve->y = y;
				//AfxMessageBox("Curve point");
				return true;
			}
		}
	}
	return false;
}

void CBezier::Draw(CDC* pDC, int index, COLORREF color)
{
	CPen pen(PS_SOLID, m_nWidth, color);
	
	pDC->SelectObject(&pen);
	pDC->PolyBezier(&m_lpPoints[index], 4);
}

void CBezier::Draw(CDC* pDC)
{
	CBrush br(RGB(0, 0, 0));
	RECT rect;
	
	CPen* pOldPen = pDC->SelectObject(&m_curvePen);
	pDC->PolyBezier(m_lpPoints, m_nAmount);

	if (m_mode != VIEW) {
		for (int i = m_bClosed ? 3 : 0; i < m_nAmount ; i += 3) {
			DoRect(&rect, m_lpPoints[i].x - HSZ, m_lpPoints[i].y - HSZ, HSZ * 2, HSZ * 2);
			pDC->FrameRect(&rect, &br);
		}
	}

	pDC->SelectObject(pOldPen);

	if (m_bShowHandles && m_mode == EDIT) {
		RECT rect;
		int i = m_nActiveCtrl;

		DoRect(&rect,m_lpPoints[i].x-HSZ,m_lpPoints[i].y-HSZ, HSZ * 2, HSZ * 2);
		pDC->FillRect( &rect, &br);

		if (m_bClosed && (i == 0 || i == m_nAmount - 1)) {
			DrawHandle(1, pDC);
			DrawHandle(m_nAmount - 2, pDC);
		}
		else {
			if (i < m_nAmount - 1) 
				DrawHandle(i + 1, pDC);
			if (i > 0)
				DrawHandle(i - 1, pDC);
		}
	}
	/*else if (m_lpPoints) {
		DoRect(&rect,m_lpPoints[m_nIndexCaptured].x-HSZ,m_lpPoints[m_nIndexCaptured].y-HSZ, HSZ * 2, HSZ * 2);
		pDC->FillRect( &rect, &br);
	}*/
}

void CBezier::DrawHandle(int nIndex, CDC* pDC)
{
	if (m_bClosed) {
		if (nIndex < 0) nIndex = m_nAmount + nIndex;
		else if (nIndex >= m_nAmount) nIndex = m_nAmount - nIndex;
	}
	
	CPen* pOldPen = pDC->SelectObject(&m_handlePen);
	int nRem = nIndex % 3;

	pDC->MoveTo(m_lpPoints[nIndex]);
	
	if (nRem == 1) 
		pDC->LineTo(m_lpPoints[nIndex - 1]);
	else if (nRem == 2) 
		pDC->LineTo(m_lpPoints[nIndex + 1]);
	else {
		TRACE("Attempt to draw a control as a handle");
		return;
	}
	
	RECT rect;
	DoRect(&rect,m_lpPoints[nIndex].x - HSZ,m_lpPoints[nIndex].y - HSZ, HSZ * 2, HSZ * 2);
	pDC->FillRect( &rect, &CBrush(RGB(0, 0, 255)));

	pDC->SelectObject(pOldPen);
}

void CBezier::Handles(CDC* pDC)
{
	CPen* pOldPen = pDC->SelectObject(&m_handlePen);
	CBrush br(RGB(0, 0, 255));
	RECT rect;
	
	for (int i=0, max=m_nAmount-1; i < max ; ) {

		pDC->MoveTo(m_lpPoints[i]);
		pDC->LineTo(m_lpPoints[++i]);
		
		DoRect(&rect,m_lpPoints[i].x-HSZ,m_lpPoints[i].y-HSZ, HSZ * 2, HSZ * 2);
		pDC->FillRect( &rect, &br);
		i++;
		DoRect(&rect,m_lpPoints[i].x-HSZ,m_lpPoints[i].y-HSZ, HSZ * 2, HSZ * 2);
		pDC->FillRect( &rect, &br);

		pDC->MoveTo(m_lpPoints[i]);
		pDC->LineTo(m_lpPoints[++i]);
	}

	pDC->SelectObject(pOldPen);
}

bool CBezier::OnMouseClick(const CPoint& pt)
{
	int i;
	double u;
	CPoint onCurve;
					
	m_rgn.CombineRgn(&m_rgn, &m_rgn, RGN_DIFF);	//empty the region

	switch (m_mode) {
		case VIEW:	
					if (IsCurvePoint(pt, &i, &u, &onCurve)) {
						m_bCaptured = true;
						m_nTypeCaptured = CURVE;
						m_nIndexCaptured = i;
						m_lPointCaptured = onCurve;
						return true;
					}
					break;
		case EDIT: 
					if (m_bShowHandles && IsHandlePoint(pt, &i)) {
						if (m_nActiveCtrl == 0 || m_nActiveCtrl == m_nAmount - 1) {
							if (i != 1 && i != m_nAmount - 2) return false;
						}
						else if (i != m_nActiveCtrl - 1 && i != m_nActiveCtrl + 1) {
							return false;
						}
					
						m_bCaptured = true;
						m_nTypeCaptured = HANDLE;
						m_nIndexCaptured = i;
						return true;
					}
					else if (IsControlPoint(pt, &i)) {
						m_bCaptured = true;
						m_nTypeCaptured = CONTROL;
						m_nIndexCaptured = i;

						if (m_bShowHandles) {
							AddHandleRgn (GetSafeIndex(m_nActiveCtrl - 1));
							AddHandleRgn (GetSafeIndex(m_nActiveCtrl + 1));
						}
						else {
							m_bShowHandles = true;
						}
						m_nActiveCtrl = i;
						AddHandleRgn(GetSafeIndex(i - 1));
						AddHandleRgn(GetSafeIndex(i + 1));
						return true;
					}
					else if (m_bShowHandles) {
						AddHandleRgn (GetSafeIndex(m_nActiveCtrl - 1));
						AddHandleRgn (GetSafeIndex(m_nActiveCtrl + 1));
						m_bShowHandles = false;
						return true;
					}
					break;
		case ADD: 
					if (IsCurvePoint(pt, &i, &u, &onCurve) 
						&& !IsControlPoint(pt, &i)) {
						AddCurveRgn(i);
						if (AddControlPoint(onCurve, i, u)) {
							AddCurveRgn(i);
							AddCurveRgn(i + 3);
							return true;
						}
					}
					break;
		case DEL: 
					if (IsControlPoint(pt, &i) && !(m_bClosed && 
						GetCtrlPointNumber() <= 2)) {
						AddCurveRgn(i - 3);
						AddCurveRgn(i);
						if (DelControlPoint(pt, i)) {
							AddCurveRgn(i - 3);
							return true;
						}
					}
					break;
	}
	return false;
}

void CBezier::OnMoveCaptured(CPoint pt)
{
	if (m_bCaptured == true) {
		CSize sz;
		CRgn rgn;
		int i = m_nIndexCaptured;
		int maxx, maxy, minx, miny;

		m_rgn.CombineRgn(&m_rgn, &m_rgn, RGN_DIFF); // it deletes current region

		switch (m_nTypeCaptured) {
		
			case CONTROL: 
						 sz = pt - m_lpPoints[i];
						 if (m_bClosed && (i == 0 || i == m_nAmount - 1)) {
							 /* Add modified regions */
							 AddCurveRgn (0);				// first curve
							 AddCurveRgn (m_nAmount - 3);	// last curve
							 AddHandleRgn (1);				// first handle
							 AddHandleRgn (m_nAmount - 2);	// last handle

							 /* Move points */
							 m_lpPoints[1] += sz;				// first handle
							 m_lpPoints[m_nAmount - 2] += sz;	// last handle
							 *m_lpPoints = pt;					// first point
							 m_lpPoints[m_nAmount - 1] = pt;	// last point

							 /* Add new modified regions */
							 AddCurveRgn (0);				// first curve
							 AddCurveRgn (m_nAmount - 3);	// last curve
							 AddHandleRgn (1);				// first handle
							 AddHandleRgn (m_nAmount - 2);	// last handle
						 }
						 else {
							 /* Add modified regions */
							 AddCurveRgn (i);		// currect curve
							 AddCurveRgn (i - 3);	// previous curve
							 AddHandleRgn (i - 1);	// left handle
							 AddHandleRgn (i + 1);	// right handle
							 
							 /* Move points */
							 if (i > 0) m_lpPoints[i - 1] += sz;
							 if (i < m_nAmount - 1) m_lpPoints[i + 1] += sz;
							 m_lpPoints[i] = pt;

							 /* Add new modified regions */
							 AddCurveRgn (i);		// currect curve
							 AddCurveRgn (i - 3);	// previous curve
							 AddHandleRgn (i - 1);	// left handle
							 AddHandleRgn (i + 1);	// right handle
						 }
						 break;
			case HANDLE: 
						 sz = pt - m_lpPoints[i];
						 AddCurveRgn (i);
						 AddHandleRgn (i);

						 m_lpPoints[i] = pt;

						 AddCurveRgn (i);
						 AddHandleRgn (i);
						 break;
			case CURVE:
						sz = pt - m_lPointCaptured; 
						//m_lPointCaptured += sz;
						m_lPointCaptured = pt;
						maxx = minx = m_lpPoints->x;
						maxy = miny = m_lpPoints->y;
						
						for (i = 0; i < m_nAmount; i++) {
							if (maxx < m_lpPoints[i].x) maxx = m_lpPoints[i].x;
							if (minx > m_lpPoints[i].x) minx = m_lpPoints[i].x;
							if (maxy < m_lpPoints[i].y) maxy = m_lpPoints[i].y;
							if (miny > m_lpPoints[i].y) miny = m_lpPoints[i].y; 
							m_lpPoints[i] += sz;
							if (maxx < m_lpPoints[i].x) maxx = m_lpPoints[i].x;
							if (minx > m_lpPoints[i].x) minx = m_lpPoints[i].x;
							if (maxy < m_lpPoints[i].y) maxy = m_lpPoints[i].y;
							if (miny > m_lpPoints[i].y) miny = m_lpPoints[i].y;
						}
						rgn.CreateRectRgn(minx-HSZ, miny-HSZ, maxx+HSZ, maxy+HSZ);
						m_rgn.CombineRgn(&rgn, &m_rgn, RGN_OR);
		}
	}
}

void CBezier::OnReleaseCapture()
{
	m_bCaptured = false;
}


#ifdef _DEBUG
void CBezier::Dump(CDumpContext& dc) const
{
	CObject::Dump(dc);
}
#endif

/*void CBezier::CalcHipo(long hipoDesired, double m, long* dx, long* dy)
{
	double aux = hipoDesired / sqrt(m * m + 1);	//to preserve presition
	*dx = (long) aux;
	*dy = (long) (aux * m);
}*/

/* Valores de prueba */

/*m_nAmount = 7;
m_lpPoints[0].x=100;
m_lpPoints[0].y=200;
m_lpPoints[1].x=150;
m_lpPoints[1].y=300;
m_lpPoints[2].x=250;
m_lpPoints[2].y=450;
m_lpPoints[3].x=350;
m_lpPoints[3].y=250;
m_lpPoints[4].x=250;
m_lpPoints[4].y=450;
m_lpPoints[5].x=350;
m_lpPoints[5].y=300;
m_lpPoints[6].x=500;
m_lpPoints[6].y=250;*/

	/*
	px = x;
	py = y;
	bDraw = true;
	str.Format("Punto: (%d,%d)\nu: %f\nslope: (%d,%d)", x, y, u, xslope, yslope); 
	AfxMessageBox(str);

	  if (bDraw) {
		CPen* pOldPen = pDC->SelectObject(&m_handlePen);
		double m = xslope/yslope, a, b;

		b = 40 / sqrt(m * m+1);
		a = m * b;
		

		pDC->MoveTo(CPoint(px, py));
		//pDC->LineTo(CPoint(px + xslope, py + yslope));
		pDC->LineTo(CPoint(px + a, py + b));

		pDC->MoveTo(CPoint(px, py));
		//pDC->LineTo(CPoint(px - xslope, py - yslope));
		pDC->LineTo(CPoint(px - a, py - b));
		pDC->SelectObject(pOldPen);
	}*/

	/*CString str;
	str.Format("dy/dx= %f/%f\nm0 = %f\ndx0 = %l\ndy0 = %l", yslope, xslope, m0, dx0, dy0);

	AfxMessageBox(str);*/

//TYPE CBezier::TestPoint(const CPoint& pt)
//{
	/*int i;
	
	for (i = 0; i < m_nAmount && m_bCaptured == false; i++) {
		if (pt.x >= m_lpPoints[i].x - HSZ && pt.x <= m_lpPoints[i].x + HSZ &&
			pt.y >= m_lpPoints[i].y - HSZ && pt.y <= m_lpPoints[i].y + HSZ ) {
			m_bCaptured = true;
			m_nIndexCaptured = i;
			m_nOffsetCaptured = m_nIndexCaptured - (m_nIndexCaptured % 3);
			if (m_nOffsetCaptured == m_nIndexCaptured ) {
				m_nTypeCaptured = CONTROL;
				m_bShowHandles = true;
				m_nActiveCtrl = m_nIndexCaptured;
			}
			else {
				m_nTypeCaptured = HANDLE;
			}
		}
	}

	if (!m_bCaptured) {
		double u;
		long x, y;
		CPoint *p;

		m_bShowHandles = false;

		for (i = 0, p = m_lpPoints; i < m_nAmount; i += 3, p += 3) {
			for (u=0; u < 1; u+=0.001) {
				
				x = (long) CalcPto(p->x, p[1].x, p[2].x, p[3].x, u);
				y = (long) CalcPto(p->y, p[1].y, p[2].y, p[3].y, u);

				if (pt.x >= x-2 && pt.x <= x + 2 && pt.y >= y-2 && pt.y <= y+2) {		
					m_bCaptured = true;
					m_nIndexCaptured = i;
					m_nTypeCaptured = CURVE;
					m_lPointCaptured.x = x;
					m_lPointCaptured.y = y;

					u = 1;				//salida del sub for
					i = m_nAmount;		//salida del for
				}
			}
		}
	}

	return m_bCaptured;*/
//}

/*void CBezier::ChangeSize(int n)
{
	if (n <= m_nAmount) m_nAmount = n;
	else {
		CPoint null;
		int i;
		CPoint* ptro = new CPoint[n];
		if (ptro !=NULL) {
			null.x=0;
			null.y=0;
			for ( i = 0; i < m_nAmount; i++) ptro[i] = m_lpPoints[i];
			for ( ; i < n; i++) ptro[i] = null;
			m_nAmount = n;
			delete[] m_lpPoints;
			m_lpPoints = ptro;
		}
	}
}*/


