// CBezier.h

class CBezier : public CObject
{
	DECLARE_DYNAMIC(CBezier)
	
	enum TYPE {CONTROL, CURVE, HANDLE, OTHER};
	enum MODE {VIEW, EDIT, ADD, DEL};

	int m_nWidth;
	bool m_bClosed;
	MODE m_mode;
	/* para la captura */
	
	bool m_bCaptured;
	TYPE m_nTypeCaptured;
	
	int m_nIndexCaptured;
	int m_nOffsetCaptured;

	CPoint m_lPointCaptured;
	CRect m_rect;
	CRgn m_rgn;

	/* para la edicion */
	bool m_bShowHandles;
	int m_nActiveCtrl;
	
private:
	int m_nAmount;
	CPen m_curvePen, m_handlePen;
	COLORREF m_ForeColor, m_BackColor;
	CPoint* m_lpPoints;

	//long txtxmax, txtxmin, txtymax, txtymin;

public:
	CBezier();
	~CBezier();

	void Draw(CDC*);
	void Draw(CDC* pDC, int, COLORREF);
	void Handles(CDC*);
	void DrawHandle(int nIndex, CDC* pDC);
	
	void AddCurveRgn(int);
	void AddHandleRgn(int);

	bool IsControlPoint(const CPoint& pt, int * nIndex);
	bool IsHandlePoint(const CPoint& pt, int * nIndex);
	bool IsCurvePoint(const CPoint& pt, int *nIndex, double *dU, CPoint* onCurve);
	bool IsCaptured() {return m_bCaptured;}

	double CalcPto(long, long, long, long, double);
	void CalcMinMax (long, long, long, long, long&, long&);
	double CalcSlope(long, long, long, long, double);
	void CalcHandle(int nIndex, int dc);

	bool OnMouseClick(const CPoint& pt);
	void OnMoveCaptured(CPoint);
	void OnReleaseCapture();

	void GetCurve(int pos, CPoint*, CPoint*, CPoint*);
	int GetCurveNumber() {return m_nAmount/3;}
	int GetPointNumber() {return m_nAmount;}
	int GetCtrlPointNumber() {
		return (m_nAmount) ? (1 + m_nAmount/3) : 0;
	}
	
	void SetClosed() { m_bClosed = TRUE; }
	void SetOpen() { m_bClosed = FALSE; }
	void SetViewMode() {m_mode = VIEW;}
	void SetEditMode() {m_mode = EDIT;}
	void SetAddMode() {m_mode = ADD;}
	void SetDelMode() {m_mode = DEL;}

	bool IsViewMode() {return (m_mode == VIEW);}
	bool IsEditMode() {return (m_mode == EDIT);}
	bool IsAddMode() {return (m_mode == ADD);}
	bool IsDelMode() {return (m_mode == DEL);}

	void AddPoint(long x, long y);
	bool AddControlPoint(const CPoint& pt, int nIndex, double u);
	void AddFirstPoint(long, long);
	void AddCurve(long, long, long, long, long, long);	

	bool DelControlPoint(const CPoint& pt, int nIndex);

	void Close();

	inline void DoRect(RECT*,long, long, long, long);
	CRect Area() {return m_rect;}

	bool IsValid(int nIndex) {
		return (nIndex >= 0 && nIndex < m_nAmount);
	}
	
	bool IsHandle(int nIndex) {
		return (IsValid(nIndex) && (nIndex % 3));
	}

	bool IsControl(int nIndex) {
		return (IsValid(nIndex) && !(nIndex % 3));
	}

	int GetSafeIndex(int i) {
		if (m_bClosed) {
			if (i < 0)
				i = m_nAmount - 1 + i;
			else if (i >= m_nAmount)
				i = i - m_nAmount;
		}
		ASSERT (i >= 0 && i < m_nAmount);
		return i;
	}

	bool CompareIndex(int a, int b) {
		return (
			(
				m_bClosed && (
					(a == 0 && b == m_nAmount - 1) ||
					(a == m_nAmount - 1 && b == 0) ||
					(a == b)
				)
			) || 
			(a == b)
		);
	}

	CPoint* operator[](int i) {
		return m_lpPoints + i;
	}



#ifdef _DEBUG
	void Dump(CDumpContext& dc) const;
#endif
};