#include "StdAfx.h"
#include "Image.h"

CImage::CImage()
{
	m_data = NULL;
	m_nRows = 0;
	m_nCols = 0;
}

CImage::CImage(const CImage& x)
{
	m_data = NULL;
	m_nRows = 0;
	m_nCols = 0;

	operator=(x);

	TRACE("Warning: construtor(CImage&) used");
}

CImage::CImage(int rows, int cols) 
{ 
	m_data = NULL;
	m_nRows = 0;
	m_nCols = 0;

	Create(rows, cols); 
}

CImage::CImage(CDib &dib)
{
	m_data = NULL;
	m_nRows = 0;
	m_nCols = 0;

	Create(dib.Alto(), dib.Ancho()); 
	
	operator=(dib);
}

bool CImage::Create(int rows, int cols)
{
	if (m_data) Free();

	if (m_data = new BYTE*[rows]) {
		m_nRows = rows;
		for (int i = 0; i < rows; i++) {
			if (!(m_data[i] = new BYTE[cols])) {
				m_nRows = i - 1;
				Free();
				return false;
			}
		}
		m_nCols = cols;
	}
	else 
		return false;

	return true;
}

void CImage::Free()
{
	if (m_data) 
	{
		for (int i = 0; i < m_nRows; i++)
			delete[] m_data[i];
		delete[] m_data;

		m_data = NULL;
		m_nRows = 0;
		m_nCols = 0;
	}
}

bool CImage::TestRange(int row, int col) const {
	return (row >= 0 && row < m_nRows && 
			col >= 0 && col < m_nCols);
}

void CImage::operator=(const CImage& rvalue)
{
	if(m_nRows != rvalue.m_nRows || 
		m_nCols != rvalue.m_nCols) 
	{
		if (!Create(rvalue.m_nRows, rvalue.m_nCols))
			throw;
	}

	for (int i=0; i < m_nRows; i++)
			memcpy(m_data[i], rvalue.m_data[i], m_nCols);
}

void CImage::operator=(const int c)
{
	for (int i = 0; i < GetRows(); i++)
		memset(m_data[i], c, m_nCols);
}

/* Dilation */
void CImage::operator+=(const CStrElement& elm)
{
	int row, col, i, j, drow, dcol, k;
	CImage tmp(m_nRows, m_nCols);

	tmp = WHITE;

	for (row = 0; row < m_nRows; row++)
		for (col = 0; col < m_nCols; col++)
			if (m_data[row][col] == BLACK) 
			{	
				drow = row - elm.GetOrgRow();
				dcol = col - elm.GetOrgCol();

				for (i = 0; i < elm.GetRows(); i++)
				  for (j = 0; j < elm.GetCols(); j++)
					if (tmp.TestRange(i + drow,j + dcol) 
						&& (k = elm[i][j]) >= 0)
						  //tmp[i + drow][j + dcol] |= k;
						  if (tmp[i + drow][j + dcol] == WHITE) 
							  tmp[i + drow][j + dcol] = k;
			}

	*this = tmp;
}


/* Erosion */
void CImage::operator-=(const CStrElement& elm)
{
	int row, col, i, j, drow, dcol, k, r;
	CImage tmp(m_nRows, m_nCols);

	tmp = WHITE;

	for (row = 0; row < m_nRows; row++)
		for (col = 0; col < m_nCols; col++) 
			if (m_data[row][col] == BLACK) 
			{
				drow = row - elm.GetOrgRow();
				dcol = col - elm.GetOrgCol();

				r = BLACK;
				for (i = 0; i < elm.GetRows(); i++)
					for (j = 0; j < elm.GetCols(); j++)
					{
						k = elm[i][j];
						if (tmp.TestRange(i + drow, j + dcol))
						{
							if ((k == BLACK) && (m_data[i + drow][j + dcol] == WHITE)) 
								r = WHITE;
						}
						else if (k == BLACK)  
							r = WHITE;
						}
				tmp[row][col] = r;
			}

	*this = tmp;
}
