// ImageBar.cpp: implementation of the CImageBar class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "SRAF.h"
#include <afxext.h>
#include "ImageBar.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CImageBar::CImageBar()
{
}

CImageBar::~CImageBar()
{

}

CSize CImageBar::CalcFixedLayout( BOOL bStretch, BOOL bHorz )
{
	CSize sz = CDialogBar::CalcFixedLayout(bStretch, bHorz);
	CFrameWnd* pFrame = GetDockingFrame();
	CRect rect;
	pFrame->GetClientRect(&rect);
	sz.cy = rect.bottom - 48;
	GetCtrl()->MoveWindow(3, 20, sz.cx - 6, sz.cy - 23);
	return sz;
}

bool CImageBar::Create(CWnd* pParent)
{
	if (!CDialogBar::Create(pParent, IDD_IMAGE_BAR, 
		CBRS_LEFT, IDD_IMAGE_BAR))
		return false;
	
	HBITMAP hBitmap;

	m_bmpNewGrp.LoadBitmap(IDB_NEW_GROUP);
	hBitmap = (HBITMAP) m_bmpNewGrp.GetSafeHandle();
	((CButton*) GetDlgItem(ID_NEW_GROUP))->SetBitmap(hBitmap);

	m_bmpAdd.LoadBitmap(IDB_ADD_ITEM);
	hBitmap = (HBITMAP) m_bmpAdd.GetSafeHandle();
	((CButton*) GetDlgItem(ID_ADD_ITEM))->SetBitmap(hBitmap);

	m_bmpDel.LoadBitmap(IDB_DEL_ITEM);
	hBitmap = (HBITMAP) m_bmpDel.GetSafeHandle();
	((CButton*) GetDlgItem(ID_DEL_ITEM))->SetBitmap(hBitmap);

	return true;
}

void CImageBar::Populate()
{
	m_imageList.Create(16, 16, 0, 4, 10);	//size, size, flag, initial number, max number

	m_imageList.Add(AfxGetApp()->LoadIcon(IDI_IMG_GROUP));
	m_imageList.Add(AfxGetApp()->LoadIcon(IDI_IMG_ITEM));
	m_imageList.Add(AfxGetApp()->LoadIcon(IDI_SELECTED_GROUP));
	m_imageList.Add(AfxGetApp()->LoadIcon(IDI_SELECTED_ITEM));
	
	CTreeCtrl* pTree = (CTreeCtrl*) GetDlgItem(IDC_TREE);
	pTree->SetImageList(&m_imageList, TVSIL_NORMAL);

	m_imgSet.m_strSort = "nro_grupo";

	try
	{
		m_grpSet.Open();
		m_imgSet.Open();
	}
	catch(CDaoException* e)
	{
		AfxMessageBox(e->m_pErrorInfo->m_strDescription);
		e->Delete();
		return;
	}

	//.ResetContent();
	if (m_grpSet.IsOpen() && m_imgSet.IsOpen())
	{ 
		HTREEITEM group;
		while(!m_grpSet.IsEOF())
		{
			group = InsertItem(m_grpSet.m_nombre, m_grpSet.m_nro_grupo, 0, 0);
			while (!m_imgSet.IsEOF() && 
				m_imgSet.m_nro_grupo == m_grpSet.m_nro_grupo) {
				InsertItem(m_imgSet.m_nombre, m_imgSet.m_nro_imagen, 1, 1, group);
				m_imgSet.MoveNext();
			}
			pTree->Expand(group, TVE_EXPAND);
			m_grpSet.MoveNext();
		}
	}

	m_grpSet.Close();
	m_imgSet.Close();
}

HTREEITEM CImageBar::InsertItem(CString texto, LPARAM parametro,
	int imagen, int seleccion, HTREEITEM hPadre, HTREEITEM hAnterior)
{
	TV_INSERTSTRUCT m_tvinsert;	

	// Campos comunes.
	m_tvinsert.item.mask = TVIF_IMAGE | TVIF_SELECTEDIMAGE | 
		TVIF_TEXT | TVIF_PARAM;
	m_tvinsert.item.hItem = NULL;
	m_tvinsert.item.state = 0;
	m_tvinsert.item.stateMask = 0;
	m_tvinsert.item.cchTextMax = 6;
	m_tvinsert.item.cChildren = 0;
	// fin campos comunes

	m_tvinsert.hParent = hPadre;
	m_tvinsert.hInsertAfter = hAnterior;
	m_tvinsert.item.pszText = texto.GetBuffer(0);
	m_tvinsert.item.lParam = parametro;
	m_tvinsert.item.iImage = imagen;
	m_tvinsert.item.iSelectedImage = seleccion;

	CTreeCtrl* pTree = (CTreeCtrl*) GetDlgItem(IDC_TREE);

	return pTree->InsertItem(&m_tvinsert);
}

bool CImageBar::AddGroup()
{	
	try
	{
		m_grpSet.Open();
		m_grpSet.AddNew();
		m_grpSet.m_nombre = "Grupo";
		m_grpSet.Update();
	}
	catch(CDaoException* e)
	{
		AfxMessageBox(e->m_pErrorInfo->m_strDescription);
		e->Delete();
		if (m_grpSet.IsOpen()) m_grpSet.Close();
		return false;
	}

	/* Ir al ultimo para obtener el numero puesto por el contador*/
	m_grpSet.MoveLast();
	m_grpSet.Close();

	GetCtrl()->EditLabel(InsertItem
			(m_grpSet.m_nombre, m_grpSet.m_nro_grupo, 0, 0));

	return true;
}

HTREEITEM CImageBar::AddItem()
{
	HTREEITEM group = GetCtrl()->GetSelectedItem();
	HTREEITEM item = GetCtrl()->GetParentItem(group);

	if (group == NULL) {
		AfxMessageBox("Debe seleccionar un grupo");
		return NULL;
	}
	
	// ver si es una imagen o un grupo
	if (item != NULL) group = item;
	
	try
	{
		m_imgSet.Open();
		m_imgSet.AddNew();
		m_imgSet.m_nombre = "Figura";
		m_imgSet.m_nro_grupo = GetCtrl()->GetItemData(group);
		m_imgSet.Update();
	}
	catch(CDaoException* e)
	{
		AfxMessageBox(e->m_pErrorInfo->m_strDescription);
		e->Delete();
		if (m_imgSet.IsOpen()) m_imgSet.Close();
		return NULL;
	}

	/* Ir al ultimo para obtener el numero puesto por el contador*/
	m_imgSet.MoveLast();
	m_imgSet.Close();

	item = InsertItem(m_imgSet.m_nombre, m_imgSet.m_nro_imagen, 1, 1, group);
	GetCtrl()->Expand(group, TVE_EXPAND);
	
	//GetCtrl()->EditLabel(item);

	return item;
}

BOOL CImageBar::DelItem()
{
	HTREEITEM hItem = GetCtrl()->GetSelectedItem();
	CString str;

	if (hItem == NULL) {
		AfxMessageBox("Debe seleccionar la figura o grupo a eliminar");
		return FALSE;;
	}

	if (GetCtrl()->ItemHasChildren(hItem)) {
		AfxMessageBox("El grupo debe estar vacio para ser eliminado");
		return FALSE;
	}

	str.Format("Desea eliminar: %s?", GetCtrl()->GetItemText(hItem));

	if (AfxMessageBox(str, MB_ICONQUESTION | MB_YESNO) == IDNO)
		return FALSE;

	// ver si es una imagen o un grupo
	if (GetCtrl()->GetParentItem(hItem) == NULL) {
		// Es un grupo
		m_grpSet.m_strFilter.Format("nro_grupo = %d", GetCtrl()->GetItemData(hItem));
		try {
			m_grpSet.Open();
			m_grpSet.Delete();
			m_grpSet.Close();
		}
		catch(CDaoException* e)
		{
			AfxMessageBox(e->m_pErrorInfo->m_strDescription);
			e->Delete();
			if (m_grpSet.IsOpen()) m_grpSet.Close();
			return FALSE;
		}
	}
	else {
		// Es una imagen
		m_imgSet.m_strFilter.Format("nro_imagen = %d", GetCtrl()->GetItemData(hItem));
		try {
			m_imgSet.Open();
			m_imgSet.Delete();
			m_imgSet.Close();
		}
		catch(CDaoException* e)
		{
			AfxMessageBox(e->m_pErrorInfo->m_strDescription);
			e->Delete();
			if (m_imgSet.IsOpen()) m_imgSet.Close();
			return FALSE;
		}
	}
	return GetCtrl()->DeleteItem(hItem);
}

HTREEITEM CImageBar::EditItem(LPNMHDR pnmhdr, LRESULT *pLResult)
{
	TV_DISPINFO		*ptvinfo;

	//((CTreeCtrlPage *)GetParent())->ShowNotification(pnmhdr, pLResult);
	ptvinfo = (TV_DISPINFO *)pnmhdr;
	if (ptvinfo->item.pszText != NULL)
	{
		CTreeCtrl* pTree = (CTreeCtrl*) GetDlgItem(IDC_TREE);
		
		ptvinfo->item.mask = TVIF_TEXT;
		pTree->SetItem(&ptvinfo->item);

		// Actualizar la base de datos
		if (pTree->GetParentItem(ptvinfo->item.hItem) == NULL) {
			// Es un grupo
			m_grpSet.m_strFilter.Format("nro_grupo = %d", ptvinfo->item.lParam);
			m_grpSet.Open();
			m_grpSet.Edit();
			m_grpSet.m_nombre = ptvinfo->item.pszText;
			m_grpSet.Update();
			m_grpSet.Close();
		}
		else {
			// Es una imagen
			m_imgSet.m_strFilter.Format("nro_imagen = %d", ptvinfo->item.lParam);
			m_imgSet.Open();
			m_imgSet.Edit();
			m_imgSet.m_nombre = ptvinfo->item.pszText;
			m_imgSet.Update();
			m_imgSet.Close();
		}
	}
	*pLResult = TRUE;

	return ptvinfo->item.hItem;
}

void CImageBar::EditLabel(HTREEITEM item)
{
	GetCtrl()->EditLabel(item);
}


void CImageBar::MarkItem(HTREEITEM hItem)
{
	int nImage, nSelectedImage;
	CTreeCtrl* pTree = GetCtrl();

	pTree->GetItemImage(hItem, nImage, nSelectedImage);

	if (IsGroup(hItem)) {
		HTREEITEM hChild = pTree->GetChildItem(hItem);

		if (nImage == GROUP) {
			while (hChild) {
				pTree->SetItemImage(hChild, MARK_ITEM, MARK_ITEM);
				hChild = pTree->GetNextSiblingItem(hChild);
			}			
			pTree->SetItemImage(hItem, MARK_GROUP, MARK_GROUP);
		}
		else {
			while (hChild) {
				pTree->SetItemImage(hChild, ITEM, ITEM);
				hChild = pTree->GetNextSiblingItem(hChild);
			}			
			pTree->SetItemImage(hItem, GROUP, GROUP);
		}
	}
	else {
		if (nImage == ITEM) {
			HTREEITEM hSibling;
			
			pTree->SetItemImage(hItem, MARK_ITEM, MARK_ITEM);

			hSibling = pTree->GetNextSiblingItem(hItem);
			while (hSibling) {
				pTree->GetItemImage(hSibling, nImage, nSelectedImage);
				if (nImage == ITEM) return; //there is at least one unselected
				hSibling = pTree->GetNextSiblingItem(hSibling);
			}
			hSibling = pTree->GetPrevSiblingItem(hItem);
			while (hSibling) {
				pTree->GetItemImage(hSibling, nImage, nSelectedImage);
				if (nImage == ITEM) return; //there is at least one unselected
				hSibling = pTree->GetPrevSiblingItem(hSibling);
			}

			pTree->SetItemImage(pTree->GetParentItem(hItem), 
				MARK_GROUP, MARK_GROUP);
		}
		else {
			HTREEITEM hParent = pTree->GetParentItem(hItem);
			pTree->SetItemImage(hItem, ITEM, ITEM);
			
			pTree->GetItemImage(hParent, nImage, nSelectedImage);	
			if (nImage == MARK_GROUP)
				pTree->SetItemImage(hParent, GROUP, GROUP);
		}
	}
}

HTREEITEM CImageBar::GetFirstMarkedItem()
{
	CTreeCtrl& tree = *GetCtrl();	
	HTREEITEM hParent, hChild;
	
	for (hParent = tree.GetNextItem(NULL, TVGN_CHILD);
		hParent; hParent = tree.GetNextSiblingItem(hParent)) 
	{
		for (hChild = tree.GetChildItem(hParent); hChild;
			hChild = tree.GetNextSiblingItem(hChild)) 
		{
				if (IsMarked(hChild))
					return hChild;
		}
	}

	return NULL;
}


HTREEITEM CImageBar::GetNextMarkedItem(HTREEITEM hChild)
{
	CTreeCtrl& tree = *GetCtrl();
	HTREEITEM hParent, hSibling;
	
	ASSERT(hChild);

	hSibling = tree.GetNextSiblingItem(hChild);
	while (hSibling) {
		if (IsMarked(hSibling)) 
			return hSibling;

		hSibling = tree.GetNextSiblingItem(hSibling);
	}

	hParent = tree.GetNextSiblingItem(tree.GetParentItem(hChild));
	for (; hParent; hParent = tree.GetNextSiblingItem(hParent)) 
	{
		for (hChild = tree.GetChildItem(hParent); hChild;
			hChild = tree.GetNextSiblingItem(hChild)) 
		{
				if (IsMarked(hChild))
					return hChild;
		}
	}

	return NULL;
}