// MainFrm.cpp : implementation of the CMainFrame class
//

#include "stdafx.h"
#include "SRAF.h"
#include <afxcview.h>
#include "Image.h"
#include "EdgeDetector.h"
#include "Canny.h"
#include "Shen.h"
#include "SRAFDoc.h"

#include "Bezier.h"
#include "BezierSet.h"
#include "BezierDoc.h"
#include "ImageBar.h"

#include "MainFrm.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


#define WM_HIDE_DRAW_BAR	WM_USER + 5

/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNAMIC(CMainFrame, CMDIFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CMDIFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_CREATE()
	ON_COMMAND(ID_NEW_GROUP, OnNewGroup)
	ON_COMMAND(ID_ADD_ITEM, OnAddItem)
	ON_COMMAND(ID_DEL_ITEM, OnDelItem)
	ON_UPDATE_COMMAND_UI(ID_VER_IMAGE_BAR, OnUpdateVerImageBar)
	ON_COMMAND(ID_VER_IMAGE_BAR, OnVerImageBar)
	ON_WM_DESTROY()
	ON_COMMAND(ID_VER_DRAWINGBAR, OnVerDrawingbar)
	ON_UPDATE_COMMAND_UI(ID_VER_DRAWINGBAR, OnUpdateVerDrawingbar)
	//}}AFX_MSG_MAP
	ON_COMMAND(ID_HELP_FINDER, CMDIFrameWnd::OnHelpFinder)
	ON_COMMAND(ID_DEFAULT_HELP, CMDIFrameWnd::OnHelpFinder)
	ON_NOTIFY(NM_DBLCLK, IDC_TREE, OnDblclk)
	ON_NOTIFY(TVN_ENDLABELEDIT, IDC_TREE, OnEditItem)
	ON_MESSAGE(WM_HIDE_DRAW_BAR, OnHideDrawBar)
	ON_NOTIFY(TVN_KEYDOWN, IDC_TREE, OnKeydownTree)
END_MESSAGE_MAP()

static UINT indicators[] =
{
	ID_SEPARATOR,           // status line indicator
	ID_INDICATOR_CAPS,
	ID_INDICATOR_NUM,
	ID_INDICATOR_SCRL,
};

const CRect CMainFrame::s_rectDefault(10,  10, 500, 400);  // static
const char CMainFrame::s_profileHeading[] = "Window size";
const char CMainFrame::s_profileRect[] = "Rect";
const char CMainFrame::s_profileIcon[] = "icon";
const char CMainFrame::s_profileMax[] = "max";
const char CMainFrame::s_profileTool[] = "tool";
const char CMainFrame::s_profileStatus[] = "status";

/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction

CMainFrame::CMainFrame()
{
	m_nChildFrame = 0;
}

CMainFrame::~CMainFrame()
{
}

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CMDIFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	if (!m_wndToolBar.Create(this) ||
		!m_wndToolBar.LoadToolBar(IDR_MAINFRAME))
	{
		TRACE0("Failed to create toolbar\n");
		return -1;      // fail to create
	}

	if (!m_wndStatusBar.Create(this) ||
		!m_wndStatusBar.SetIndicators(indicators,
		  sizeof(indicators)/sizeof(UINT)))
	{
		TRACE0("Failed to create status bar\n");
		return -1;      // fail to create
	}

	m_wndToolBar.SetBarStyle(m_wndToolBar.GetBarStyle() |
		CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC);

	m_wndToolBar.EnableDocking(CBRS_ALIGN_ANY);
	EnableDocking(CBRS_ALIGN_ANY);
	DockControlBar(&m_wndToolBar);

	if (!m_wndImgBar.Create(this)) 
	{
		TRACE0("Failed to create image bar\n");
		return -1;      // fail to create
	}	
	
	/*m_wndImgBar.SetBarStyle(m_wndImgBar.GetBarStyle() |
		CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC);*/

	m_wndImgBar.EnableDocking(CBRS_ALIGN_RIGHT | CBRS_ALIGN_LEFT);
	DockControlBar(&m_wndImgBar);

	m_wndImgBar.Populate();

	if (!m_wndDrawBar.Create(this, WS_CHILD /*| WS_VISIBLE*/ | 
		CBRS_RIGHT) || !m_wndDrawBar.LoadToolBar(IDR_DRAWTOOLS))
	{
		TRACE0("Failed to create drawing toolbar\n");
		return -1;      // fail to create
	}

	m_wndDrawBar.SetBarStyle(m_wndDrawBar.GetBarStyle() |
		CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC);

	m_wndDrawBar.EnableDocking(CBRS_ALIGN_ANY);
	EnableDocking(CBRS_ALIGN_ANY);
	DockControlBar(&m_wndDrawBar);

	FloatControlBar(&m_wndDrawBar, CPoint(650,100) , CBRS_ALIGN_RIGHT);

	return 0;
}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	return CMDIFrameWnd::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CMDIFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CMDIFrameWnd::Dump(dc);
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers


void CMainFrame::OnNewGroup() 
{
	m_wndImgBar.AddGroup();
}

void CMainFrame::OnAddItem() 
{
	HTREEITEM hItem = m_wndImgBar.AddItem();

	if (hItem) {
		POSITION curTemplatePos = AfxGetApp()->GetFirstDocTemplatePosition();

		while(curTemplatePos != NULL)
		{
			CDocTemplate* curTemplate = 
				AfxGetApp()->GetNextDocTemplate(curTemplatePos);
			CString str;
			curTemplate->GetDocString(str, CDocTemplate::docName);
			if(str == _T("Figura"))
			{
				CBezierDoc* pDoc = (CBezierDoc*) 
					curTemplate->OpenDocumentFile(NULL);
				pDoc->SetBezierID(m_wndImgBar.GetItemData(hItem));
				/*-----------------*/
				m_nChildFrame++;
				ShowControlBar(&m_wndDrawBar, TRUE, FALSE);
				/*-----------------*/
				m_wndImgBar.EditLabel(hItem);
				return;
			}
		}
		AfxMessageBox("Error en el template");
	}
}

void CMainFrame::OnDelItem() 
{
	/* Antes de eliminar el item tomar el puntero al doc */
	CBezierDoc *pDoc = FindBezierDoc(
		m_wndImgBar.GetItemData(m_wndImgBar.GetSelectedItem()));

	/* Eliminar el item y Si hay un doc abierto cerrarlo, 
	los datos se borraran por la integridad referencial */
	if (m_wndImgBar.DelItem() && pDoc != NULL)
		pDoc->OnCloseDocument();	
}

void CMainFrame::OnEditItem(LPNMHDR pnmhdr, LRESULT *pLResult)
{
	HTREEITEM hItem = m_wndImgBar.EditItem(pnmhdr, pLResult);
	CBezierDoc* pDoc = FindBezierDoc(m_wndImgBar.GetItemData(hItem));

	if (pDoc) pDoc->SetTitle(m_wndImgBar.GetItemText(hItem));
}

void CMainFrame::OnUpdateVerImageBar(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_wndImgBar.IsWindowVisible());
}

void CMainFrame::OnVerImageBar() 
{	
	ShowControlBar(&m_wndImgBar, !m_wndImgBar.IsWindowVisible(), FALSE);
}

void CMainFrame::OnDblclk(NMHDR* pNMHDR, LRESULT* pResult) 
{
	HTREEITEM hItem = m_wndImgBar.GetSelectedItem();

	*pResult = 0;
	if (m_wndImgBar.IsGroup(hItem)) return;
	
	CBezierDoc* pDoc = FindBezierDoc(m_wndImgBar.GetItemData(hItem));
	if (pDoc) {
		CView* curView;
		POSITION curViewPos = pDoc->GetFirstViewPosition();

		if (curViewPos) {
			curView = pDoc->GetNextView(curViewPos);
			
			if (curView) {
				MDIActivate(curView->GetParent());
			}
		}
	}
	else {
		CString str;
		CDocTemplate* curTemplate;
		POSITION curTemplatePos = AfxGetApp()->GetFirstDocTemplatePosition();

		while(curTemplatePos != NULL)
		{
			curTemplate = AfxGetApp()->GetNextDocTemplate(curTemplatePos);
			curTemplate->GetDocString(str, CDocTemplate::docName);

			if(str == _T("Figura"))
			{
				CBezierDoc* pDoc = (CBezierDoc*) 
					curTemplate->OpenDocumentFile(m_wndImgBar.GetItemText(hItem));
				pDoc->SetBezierID(m_wndImgBar.GetItemData(hItem));
				/*-----------------*/
				m_nChildFrame++;
				ShowControlBar(&m_wndDrawBar, TRUE, FALSE);
				/*-----------------*/
				return;
			}
		}
		TRACE("DocTemplate error");
	}
}

int CMainFrame::GetSavedState(int nCmdShow)
{
	CString strText;
	BOOL bIconic, bMaximized;
	UINT flags;
	WINDOWPLACEMENT wndpl;
	CRect rect;

	//if (m_bFirstTime) {
		//m_bFirstTime = FALSE;
		strText = AfxGetApp()->GetProfileString(s_profileHeading,
		                                        s_profileRect);
		if (!strText.IsEmpty()) {
			rect.left = atoi((const char*) strText);
			rect.top = atoi((const char*) strText + 5);
			rect.right = atoi((const char*) strText + 10);
			rect.bottom = atoi((const char*) strText + 15);
		}
		else {
			rect = s_rectDefault;
		}

		bIconic = AfxGetApp()->GetProfileInt(s_profileHeading,
		                                     s_profileIcon, 0);
		bMaximized = AfxGetApp()->GetProfileInt(s_profileHeading,
		                                        s_profileMax, 0);   
		if (bIconic) {
			nCmdShow = SW_SHOWMINNOACTIVE;
			if (bMaximized) {
				flags = WPF_RESTORETOMAXIMIZED;
			}
			else {
				flags = WPF_SETMINPOSITION;
			}
		}
		else {
			if (bMaximized) {
				nCmdShow = SW_SHOWMAXIMIZED;
				flags = WPF_RESTORETOMAXIMIZED;
			}
			else {
				nCmdShow = SW_NORMAL;
				flags = WPF_SETMINPOSITION;
			}
		}
		wndpl.length = sizeof(WINDOWPLACEMENT);
		wndpl.showCmd = nCmdShow;
		wndpl.flags = flags;
		wndpl.ptMinPosition = CPoint(0, 0);
		wndpl.ptMaxPosition =
			CPoint(-::GetSystemMetrics(SM_CXBORDER),
			       -::GetSystemMetrics(SM_CYBORDER));
		wndpl.rcNormalPosition = rect;
		//LoadBarState(AfxGetApp()->m_pszProfileName);
		// sets window's position and minimized/maximized status
		BOOL bRet = SetWindowPlacement(&wndpl);
	//}
	return nCmdShow;
}

void CMainFrame::OnDestroy() 
{
	CString strText;
	BOOL bIconic, bMaximized;

	WINDOWPLACEMENT wndpl;
	wndpl.length = sizeof(WINDOWPLACEMENT);
	// gets current window position and
	//  iconized/maximized status
	BOOL bRet = GetWindowPlacement(&wndpl);
	if (wndpl.showCmd == SW_SHOWNORMAL) {
		bIconic = FALSE;
		bMaximized = FALSE;
	}
	else if (wndpl.showCmd == SW_SHOWMAXIMIZED) {
		bIconic = FALSE;
		bMaximized = TRUE;
	} 
	else if (wndpl.showCmd == SW_SHOWMINIMIZED) {
		bIconic = TRUE;
		if (wndpl.flags) {
			bMaximized = TRUE;
		}
		else {
			bMaximized = FALSE;
		}
	}
	strText.Format("%04d %04d %04d %04d",
	               wndpl.rcNormalPosition.left,
	               wndpl.rcNormalPosition.top,
	               wndpl.rcNormalPosition.right,
	               wndpl.rcNormalPosition.bottom);
	AfxGetApp()->WriteProfileString(s_profileHeading,
	                                s_profileRect, strText);
	AfxGetApp()->WriteProfileInt(s_profileHeading,
	                             s_profileIcon, bIconic);
	AfxGetApp()->WriteProfileInt(s_profileHeading,
	                             s_profileMax, bMaximized);
	//SaveBarState(AfxGetApp()->m_pszProfileName);
	
	CMDIFrameWnd::OnDestroy();
	
	// TODO: Add your message handler code here
	
}

void CMainFrame::OnVerDrawingbar() 
{
	ShowControlBar(&m_wndDrawBar, !m_wndDrawBar.IsWindowVisible(), FALSE);
}

void CMainFrame::OnUpdateVerDrawingbar(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_wndDrawBar.IsWindowVisible());
}

void CMainFrame::OnHideDrawBar(WPARAM wParam, LPARAM lParam)
{
	if (--m_nChildFrame == 0)
		ShowControlBar(&m_wndDrawBar, FALSE, FALSE);
}


CBezierDoc* CMainFrame::FindBezierDoc(long lID)
{
	CString str;
	POSITION curTemplatePos = AfxGetApp()->GetFirstDocTemplatePosition();
	CDocTemplate* curTemplate;

	while (curTemplatePos) {
		curTemplate = AfxGetApp()->GetNextDocTemplate(curTemplatePos);
		curTemplate->GetDocString(str, CDocTemplate::docName);
		if(str == _T("Figura")) {
			CBezierDoc* curDoc;
			POSITION curDocPos = curTemplate->GetFirstDocPosition();
			while (curDocPos) {
				curDoc = (CBezierDoc*) curTemplate->GetNextDoc(curDocPos);
				if (curDoc->GetBezierID() == lID)
					return curDoc;
			}
		}
	}
	return NULL;
}

void CMainFrame::OnKeydownTree(NMHDR* pNMHDR, LRESULT* pResult) 
{
	TV_KEYDOWN* pTVKeyDown = (TV_KEYDOWN*)pNMHDR;

	if (pTVKeyDown->wVKey == 32) 
		m_wndImgBar.MarkItem(m_wndImgBar.GetSelectedItem());
	/*else {
		HTREEITEM hItem = m_wndImgBar.GetFirstMarkedItem();

		while (hItem) {
			AfxMessageBox(m_wndImgBar.GetItemText(hItem));
			hItem = m_wndImgBar.GetNextMarkedItem(hItem);
		}
	}*/

	
	*pResult = 1;
}