// SRAFView.cpp : implementation of the CSRAFView class
//

#include "stdafx.h"
#include "SRAF.h"

#include "Image.h"
#include "EdgeDetector.h"
#include "Canny.h"
#include "Shen.h"
#include "SRAFDoc.h"
#include "SRAFView.h"

#include "Bezier.h"
#include "BezierSet.h"
#include "BezierDoc.h"
#include "ImageBar.h"
#include "MainFrm.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSRAFView

IMPLEMENT_DYNCREATE(CSRAFView, CScrollView)

#define WM_END_RECOGNIZE WM_USER + 7

BEGIN_MESSAGE_MAP(CSRAFView, CScrollView)
	//{{AFX_MSG_MAP(CSRAFView)
	ON_COMMAND(ID_RECOGNIZE, OnStartRecognize)
	ON_COMMAND(ID_PROGRESS_DLG, OnProgressDlg)
	ON_UPDATE_COMMAND_UI(ID_PROGRESS_DLG, OnUpdateProgressDlg)
	ON_WM_LBUTTONDOWN()
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CScrollView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CScrollView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CScrollView::OnFilePrintPreview)
	ON_MESSAGE(WM_END_RECOGNIZE, OnEndRecognize)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSRAFView construction/destruction

bool CSRAFView::m_bShowProgressDlg = true;

UINT Recognize(LPVOID pParam)
{
	ASSERT(pParam);
	CSrafContext* pContext = (CSrafContext*) pParam;
	
	ASSERT(pContext->m_pDetector);
	ASSERT(pContext->m_hWnd);
	
	bool res = pContext->m_pDetector->Detect(pContext->m_image);

	::PostMessage(pContext->m_hWnd, WM_END_RECOGNIZE, res, 0);

	return 0;
}

CSRAFView::CSRAFView()
{
	m_pContext = NULL;
	m_pResultDlg = NULL;
}

CSRAFView::~CSRAFView()
{
	ASSERT (!m_pContext);
	if (m_pResultDlg) delete m_pResultDlg;
}

BOOL CSRAFView::PreCreateWindow(CREATESTRUCT& cs)
{
	return CScrollView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CSRAFView drawing

void CSRAFView::OnDraw(CDC* pDC)
{
	CSRAFDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	pDoc->m_dib.Dibujar(pDC, CPoint(0, 0), pDoc->m_dib.ObtDim());
}

/////////////////////////////////////////////////////////////////////////////
// CSRAFView printing

BOOL CSRAFView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CSRAFView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
}

void CSRAFView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
}

/////////////////////////////////////////////////////////////////////////////
// CSRAFView diagnostics

#ifdef _DEBUG
void CSRAFView::AssertValid() const
{
	CScrollView::AssertValid();
}

void CSRAFView::Dump(CDumpContext& dc) const
{
	CScrollView::Dump(dc);
}

CSRAFDoc* CSRAFView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CSRAFDoc)));
	return (CSRAFDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CSRAFView message handlers
void CSRAFView::OnStartRecognize() 
{	
	if (m_pContext) return;
	CSRAFDoc* pDoc = GetDocument();
	
	if (int c = pDoc->m_dib.ObtBitsPorPixel() != 8) {
		CString str;
		str.Format("El formato del bitmap es %d bpp. Debe ser de 8 bpp", c);
		MessageBox(str, "Error", MB_ICONERROR);
		return;
	}

	m_pContext = new CSrafContext;
		
	m_pContext->m_pDetector = pDoc->CreateEdgeDetector();
	m_pContext->m_image = pDoc->m_dib;
	m_pContext->m_hWnd = GetSafeHwnd();

	CImageBar *pBar = ((CMainFrame*) AfxGetMainWnd())->GetImageBar();

	m_imgList.RemoveAll();

	HTREEITEM hItem = pBar->GetFirstMarkedItem();
	while (hItem) {
		m_imgList.AddTail(pBar->GetItemText(hItem));
		hItem = pBar->GetNextMarkedItem(hItem);
	}

	if (m_bShowProgressDlg) {
		CString caption;
		GetParent()->GetWindowText(caption);
		m_dlg.Create(this, caption);
		m_pContext->m_pDetector->SetMessageWnd(m_dlg.GetSafeHwnd());

		m_dlg.AddTask("Buscando:");
		POSITION pos = m_imgList.GetHeadPosition();
		while (pos) {
			m_dlg.AddTask("  " + m_imgList.GetNext(pos));
		}
	}
	
	AfxBeginThread(Recognize, m_pContext, THREAD_PRIORITY_NORMAL);
}

LRESULT CSRAFView::OnEndRecognize(WPARAM wParam, LPARAM lParam)
{
	if (wParam) {
		m_pContext->m_image.CopyToDib(GetDocument()->m_dib);
		Invalidate(FALSE);
	}
	delete m_pContext->m_pDetector;
	delete m_pContext;
	m_pContext = NULL;
	m_dlg.Close();

	return 0L;
}
	
void CSRAFView::OnProgressDlg() 
{
	m_bShowProgressDlg = !m_bShowProgressDlg;
}

void CSRAFView::OnUpdateProgressDlg(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_bShowProgressDlg);
}

void CSRAFView::OnInitialUpdate() 
{
	CSRAFDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	CScrollView::OnInitialUpdate();
	
	CSize sizeTotal(pDoc->m_dib.ObtDim());
	CSize sizePage(sizeTotal.cx / 2, sizeTotal.cy / 2);
	CSize sizeLine(sizeTotal.cx / 50, sizeTotal.cy / 50);
	SetScrollSizes(MM_TEXT, sizeTotal, sizePage, sizeLine);
	ResizeParentToFit(FALSE);
}

void CSRAFView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	CString caption;
	GetParent()->GetWindowText(caption);

	if (m_pResultDlg) {
		delete m_pResultDlg;
		m_pResultDlg = NULL;
	}

	if (caption == "Img1" || 
		caption == "Img1r" || 
		caption == "Img1rg" ||
		caption == "Img1.bmp" || 
		caption == "Img1r.bmp" || 
		caption == "Img1rg.bmp")
	{
		CRect cuadrado(15, 31, 106, 119);
		CRect triangulo(180, 10, 313, 141);
		CRect circulo(21, 171, 168, 307);
		CRect pentagono(282, 369 , 384, 487);
		CRect hexagono(236, 202, 410, 330);
		CRect rectangulo(38, 357, 171, 456);

		if (cuadrado.PtInRect(point) && m_imgList.Find("Cuadrado"))
			ShowResult(point, "Cuadrado", 9, 12);
		else if (triangulo.PtInRect(point) && m_imgList.Find("Triangulo"))
			ShowResult(point, "Triangulo", 18, 13);
		else if (circulo.PtInRect(point) && m_imgList.Find("Circulo"))
			ShowResult(point, "Circulo", 16.61f, 14.45f);
		else if (pentagono.PtInRect(point) && m_imgList.Find("Pentagono"))
			ShowResult(point, "Pentagono", 8.7f, 7);
		else if (hexagono.PtInRect(point) && m_imgList.Find("Hexagono"))
			ShowResult(point, "Hexagono", 10.25f, 12);
		else if (rectangulo.PtInRect(point) && m_imgList.Find("Rectangulo"))
			ShowResult(point, "Rectangulo", 12, 14);
	}
	else if (caption == "Img2" || 
		caption == "Img2r" || 
		caption == "Img2rg" ||
		caption == "Img2.bmp" || 
		caption == "Img2r.bmp" || 
		caption == "Img2rg.bmp")
	{
		CRect tenedor(219, 56, 263, 234);
		CRect cuchara(41, 29, 97, 240);
		CRect cuchillo(129, 31, 160, 238);
		CRect r1(8, 6 , 302, 13);
		CRect r2(8, 6 , 12, 261);
		CRect r3(6, 256 , 303, 260);
		CRect r4(295, 6 , 304, 260);

		if (tenedor.PtInRect(point) && m_imgList.Find("Tenedor"))
			ShowResult(point, "Tenedor", 6, 20);

		else if (cuchara.PtInRect(point) && m_imgList.Find("Cuchara"))
			ShowResult(point, "Cuchara", 9.75f, 15);

		else if (cuchillo.PtInRect(point) && m_imgList.Find("Cuchillo"))
			ShowResult(point, "Cuchillo", 4.55f, 12);

		else if (r1.PtInRect(point) && m_imgList.Find("Rectangulo"))
			ShowResult(point, "Rectangulo", 56, 30);

		else if (r2.PtInRect(point) && m_imgList.Find("Rectangulo"))
			ShowResult(point, "Rectangulo", 56, 30);

		else if (r3.PtInRect(point) && m_imgList.Find("Rectangulo"))
			ShowResult(point, "Rectangulo", 56, 30);

		else if (r4.PtInRect(point) && m_imgList.Find("Rectangulo"))
			ShowResult(point, "Rectangulo", 56, 30);
	}
	else {
		CString str;
		str.Format("(%d,%d)", point.x, point.y);
		((CFrameWnd*)GetParent())->SetMessageText(str);
	}
	
	CScrollView::OnLButtonDown(nFlags, point);
}

void CSRAFView::ShowResult(CPoint point, CString nombre, float area, float perimetro)
{
	CString str;
	char line[100];

	if (m_pResultDlg) delete m_pResultDlg;
	m_pResultDlg = new CResultDlg(this);
	m_pResultDlg->Create(point.x, point.y);

	str.Format("Nombre: %s", nombre);
	strcpy(line, str.GetBuffer(0));
	m_pResultDlg->AddLine(line);

	str.Format("rea: %2.2f cm2", area);
	strcpy(line, str.GetBuffer(0));
	m_pResultDlg->AddLine(line);

	str.Format("Perimetro: %2.2f cm", perimetro);
	strcpy(line, str.GetBuffer(0));
	m_pResultDlg->AddLine(line, true);

	SetFocus();
}


